function CLSW_analysis

% ---------------------------------------------------
% Code by Caio Waisman, 2015
% ---------------------------------------------------


clear all
close all
clc

tic

% Getting and sorting data
data_=xlsread('CLSW_data.xls');

data = [data_(:,11) data_(:,1) data_(:,9) data_(:,6) data_(:,5) data_(:,4) ...
    data_(:,2) data_(:,3) data_(:,7) data_(:,8) data_(:,12)];

clear data_

data=data(data(:,3)<500,:);
data=data(data(:,1)<9,:);

% Grids
N_grid = (min(data(:,1)):max(data(:,1)))';
% r_grid = (0:0.01:500)';
r_grid = [(0:1:59)';(59.01:0.01:63)';(64:1:71)';(71.01:0.01:74)';(75:1:80)';...
    (80.01:0.01:84)';(85:1:89)';(89.01:0.01:92)';(93:1:177)';(177.01:0.01:178)';...
    (179:1:212)';(212.01:0.01:215)';(216:1:220)';(220.01:0.01:225)';(226:1:236)';...
    (236.01:0.01:239)';(240:1:244)';(244.01:0.01:270)';(271:1:500)'];
tol = 1e-10;
aux = unique(data((data(:,1)==max(N_grid)),11));  

% Conditional results

% Kernel and bandwith
r = 2; % order of the Kernel
s=25; 
b = (2*s^5-(2/3)*s^2*(s^3-(-s)^3)+(1/5)*(s^5-(-s)^5))^(-1);

X = data(:,4:8);

[row col] = size(X);
x = median(X);

h = ((length(data))^(-1/(col+2*r-1)))*ones(1,5);

v0=50;
psi = ((X-kron(ones(row,1),x))./kron(ones(row,1),var(X).^0.5))./(kron(ones(row,1),h));
ind_ker = (b*((s^2*ones(row,col)-psi.^2).^2)).*(abs(psi)<=s);
prod_ker = cumprod(ind_ker,2);
K = prod_ker(:,col); 

% Bounds

F_trans = zeros(length(r_grid),length(N_grid)); % empirical cdfs of transaction prices
Fn_IPV = zeros(length(r_grid),length(N_grid)); % empirical CDF under IPV
Fn_low = zeros(length(r_grid),length(N_grid)-1); % AGQ lower bound
Fn_low2 = zeros(length(r_grid),length(N_grid)-1); % CLS lower bound
Tn_hat = zeros(length(r_grid),length(N_grid)); % used for calculations
Fn_up = zeros(length(r_grid),length(N_grid)-1); % AGQ upper bound
phi_matrix = zeros(length(r_grid),length(N_grid));

for i=1:length(r_grid)
    for j=1:length(N_grid)
        F_trans(i,j) = sum((data(:,2)<=r_grid(i)).*K.*(data(:,1)==N_grid(j)))/ ...
                sum(K.*(data(:,1)==N_grid(j)));
        Tn_hat(i,j) = sum(max(r_grid(i),data(:,2)).*K.*...
            (data(:,1)==N_grid(j)))/ sum(K.*(data(:,1)==N_grid(j)));
        phi_matrix(i,j) = func1(tol,N_grid(j),F_trans(i,j));
        Fn_IPV(i,j) = phi_matrix(i,j)^N_grid(j);
    end
end

phi_vector = zeros(length(r_grid),1);
for i=1:length(r_grid)
    for j=1:length(aux)
        F_cond = sum((data(:,1)==max(N_grid)  & data(:,11)==aux(j)).*K.*...
            (data(:,2)<=r_grid(i)))/sum(K.*(data(:,1)==max(N_grid) & data(:,11)==aux(j)));
        phi_aux = func1(tol,max(N_grid),F_cond);
        phi_vector(i) =  phi_vector(i)+(length(data(data(:,1)==max(N_grid)...
            & data(:,11)==aux(j))))/(length(data(data(:,1)==max(N_grid))))*...
            phi_aux^max(N_grid);
    end
end

for j=1:length(N_grid)-1
    for k=j+1:length(N_grid)
    Fn_low(:,j) = Fn_low(:,j)+(N_grid(j)/(N_grid(k)*(N_grid(k)-1)))...
        *F_trans(:,k);
    end
	Fn_up(:,j) =  Fn_low(:,j);  
	Fn_low2(:,j) = Fn_low(:,j) + (N_grid(j)/max(N_grid))*phi_vector;
    Fn_low(:,j) = Fn_low(:,j) + (N_grid(j)/max(N_grid))*Fn_IPV(:,length(N_grid));
    Fn_up(:,j) = Fn_up(:,j) + (N_grid(j)/max(N_grid))*F_trans(:,length(N_grid));
end

% Comparing profits

% Expected profits under IPV
pi_IPV = Tn_hat-v0*ones(length(r_grid),length(N_grid))-...
    kron((r_grid-v0*ones(length(r_grid),1)),ones(1,length(N_grid))).*Fn_IPV;

% AGQ lower bound on expected profits
pi_low = Tn_hat(:,1:length(N_grid)-1)-v0*ones(length(r_grid),length(N_grid)-1)-...
    kron((r_grid-v0*ones(length(r_grid),1)),ones(1,length(N_grid)-1)).*Fn_up;

% AGQ upper bound on expected profits
pi_up = Tn_hat(:,1:length(N_grid)-1)-v0*ones(length(r_grid),length(N_grid)-1)-...
    kron((r_grid-v0*ones(length(r_grid),1)),ones(1,length(N_grid)-1)).*Fn_low;

% CLSW upper bound on expected profits
pi_up2 = Tn_hat(:,1:length(N_grid)-1)-v0*ones(length(r_grid),length(N_grid)-1)-...
    kron((r_grid-v0*ones(length(r_grid),1)),ones(1,length(N_grid)-1)).*Fn_low2;

% Difference betweem the upper bounds (counterfactual exericse)
pi_cf = pi_up - pi_up2;

% Bounds on reserve prices

bounds_pi = zeros(6,4);
bounds_r = zeros(6,2);
bounds_r2 = zeros(6,2);
bounds_pi_IPV = zeros(6,4);
r_IPV = zeros(6,1);

for j=1:6
    bounds_pi(j,1) = pi_IPV(v0,j);
    bounds_pi(j,2) = max(pi_low(:,j));
    bounds_pi(j,3) = max(pi_up(:,j));
	bounds_pi(j,4) = max(pi_up2(:,j));
    
    % Creates inputs for table 2
    aux = r_grid(pi_up(:,j)>=bounds_pi(j,2));
    bounds_r(j,1) = aux(1,1);
    bounds_r(j,2) = max(r_grid(pi_up(:,j)>=bounds_pi(j,2)));
    
	aux = r_grid(pi_up2(:,j)>=bounds_pi(j,2));
    bounds_r2(j,1) = aux(1,1);
    bounds_r2(j,2) = max(r_grid(pi_up2(:,j)>=bounds_pi(j,2)));
    
    [bounds_pi_IPV(j,1) loc_ipv] = max(pi_IPV(:,j));
    bounds_pi_IPV(j,2) = pi_low(loc_ipv,j);
    bounds_pi_IPV(j,3) = pi_up(loc_ipv,j);
	bounds_pi_IPV(j,4) = pi_up2(loc_ipv,j);
    r_IPV(j,1) = r_grid(loc_ipv);
end

toc

% Boostrapping
tic
rng(373)
B = 200;

Fn_IPVb = zeros(length(r_grid),length(N_grid),B); 
Fn_lowb = zeros(length(r_grid),length(N_grid)-1,B); 
Fn_low2b = zeros(length(r_grid),length(N_grid)-1,B);
Fn_upb = zeros(length(r_grid),length(N_grid)-1,B);
pi_IPVb = zeros(length(r_grid),length(N_grid),B);
pi_lowb = zeros(length(r_grid),length(N_grid)-1,B); 
pi_up2b = zeros(length(r_grid),length(N_grid)-1,B);
pi_upb = zeros(length(r_grid),length(N_grid)-1,B);
pi_cfb = zeros(length(r_grid),length(N_grid)-1,B);
bounds_pib = zeros(6,4,B);
bounds_rb = zeros(6,2,B);
bounds_r2b = zeros(6,2,B);
bounds_pi_IPVb = zeros(6,4,B);
r_IPVb = zeros(6,1,B);

aux = unique(data((data(:,1)==max(N_grid)),11));  

for b=1:B
    datab=[];
    Kb=[];
    for j=1:length(N_grid)
        data_aux = data(data(:,1)==N_grid(j),:);
        K_aux = K(data(:,1)==N_grid(j));
        samp = ceil(length(data_aux)*rand(length(data_aux),1));
        Kb = [Kb;K_aux(samp)];
        datab = [datab;data_aux(samp,:)];
    end
    
    F_transb = zeros(length(r_grid),length(N_grid));  
    Tn_hatb = zeros(length(r_grid),length(N_grid)); 
    phi_matrixb = zeros(length(r_grid),length(N_grid));

    for i=1:length(r_grid)
        for j=1:length(N_grid)
            F_transb(i,j) = sum((datab(:,2)<=r_grid(i)).*Kb.*(datab(:,1)==N_grid(j)))/ ...
                    sum(Kb.*(datab(:,1)==N_grid(j)));
            Tn_hatb(i,j) = sum(max(r_grid(i),datab(:,2)).*Kb.*...
                (datab(:,1)==N_grid(j)))/ sum(Kb.*(datab(:,1)==N_grid(j)));
            phi_matrixb(i,j) = func1(tol,N_grid(j),F_transb(i,j));
            Fn_IPVb(i,j,b) = phi_matrixb(i,j)^N_grid(j);
        end
    end

    phi_vectorb = zeros(length(r_grid),1);
    for i=1:length(r_grid)
        for j=1:length(aux)
            F_condb = sum((datab(:,1)==max(N_grid)  & datab(:,11)==aux(j)).*Kb.*...
                (datab(:,2)<=r_grid(i)))/sum(Kb.*(datab(:,1)==max(N_grid) & datab(:,11)==aux(j)));
            phi_auxb = func1(tol,max(N_grid),F_condb);
            phi_vectorb(i) =  phi_vectorb(i)+(length(datab(datab(:,1)==max(N_grid)...
                & datab(:,11)==aux(j))))/(length(datab(datab(:,1)==max(N_grid))))*...
                phi_auxb^max(N_grid);
        end
    end

    for j=1:length(N_grid)-1
        for k=j+1:length(N_grid)
        Fn_lowb(:,j,b) = Fn_lowb(:,j,b)+(N_grid(j)/(N_grid(k)*(N_grid(k)-1)))...
            *F_transb(:,k);
        end
        Fn_upb(:,j,b) =  Fn_lowb(:,j,b);  
        Fn_low2b(:,j,b) = Fn_lowb(:,j,b) + (N_grid(j)/max(N_grid))*phi_vectorb;
        Fn_lowb(:,j,b) = Fn_lowb(:,j,b) + (N_grid(j)/max(N_grid))*Fn_IPVb(:,length(N_grid),b);
        Fn_upb(:,j,b) = Fn_upb(:,j,b) + (N_grid(j)/max(N_grid))*F_transb(:,length(N_grid));
    end


    pi_IPVb(:,:,b) = Tn_hatb-v0*ones(length(r_grid),length(N_grid))-...
        kron((r_grid-v0*ones(length(r_grid),1)),ones(1,length(N_grid))).*Fn_IPVb(:,:,b);

    pi_lowb(:,:,b) = Tn_hatb(:,1:length(N_grid)-1)-v0*ones(length(r_grid),length(N_grid)-1)-...
        kron((r_grid-v0*ones(length(r_grid),1)),ones(1,length(N_grid)-1)).*Fn_upb(:,:,b);

    pi_upb(:,:,b) = Tn_hatb(:,1:length(N_grid)-1)-v0*ones(length(r_grid),length(N_grid)-1)-...
        kron((r_grid-v0*ones(length(r_grid),1)),ones(1,length(N_grid)-1)).*Fn_lowb(:,:,b);

    pi_up2b(:,:,b) = Tn_hatb(:,1:length(N_grid)-1)-v0*ones(length(r_grid),length(N_grid)-1)-...
        kron((r_grid-v0*ones(length(r_grid),1)),ones(1,length(N_grid)-1)).*Fn_low2b(:,:,b); 
    
    pi_cfb = pi_upb - pi_up2b;

    for j=1:6
        bounds_pib(j,1,b) = pi_IPVb(v0,j,b);
        bounds_pib(j,2,b) = max(pi_lowb(:,j,b));
        bounds_pib(j,3,b) = max(pi_upb(:,j,b));
        bounds_pib(j,4,b) = max(pi_up2b(:,j,b));

        auxb = r_grid(pi_upb(:,j,b)>=bounds_pib(j,2,b));
        bounds_rb(j,1,b) = auxb(1,1);
        bounds_rb(j,2,b) = max(r_grid(pi_upb(:,j,b)>=bounds_pib(j,2,b)));

        auxb = r_grid(pi_up2b(:,j,b)>=bounds_pib(j,2,b));
        bounds_r2b(j,1,b) = auxb(1,1);
        bounds_r2b(j,2,b) = max(r_grid(pi_up2b(:,j,b)>=bounds_pib(j,2,b)));

        [bounds_pi_IPVb(j,1,b) loc_ipv] = max(pi_IPVb(:,j,b));
        bounds_pi_IPVb(j,2,b) = pi_lowb(loc_ipv,j,b);
        bounds_pi_IPVb(j,3,b) = pi_upb(loc_ipv,j,b);
        bounds_pi_IPVb(j,4,b) = pi_up2b(loc_ipv,j,b);
        r_IPVb(j,1,b) = r_grid(loc_ipv);
    end

end

% Getting confidence intervals
alpha = 0.05;
IC_F_IPV_low = quantile(Fn_IPVb,alpha/2,3);
IC_F_IPV_up = quantile(Fn_IPVb,1-(alpha/2),3);
IC_F_up_low = quantile(Fn_upb,alpha/2,3);
IC_F_up_up = quantile(Fn_upb,1-(alpha/2),3);
IC_F_low_low = quantile(Fn_lowb,alpha/2,3);
IC_F_low_up = quantile(Fn_lowb,1-(alpha/2),3);
IC_F_low2_low = quantile(Fn_low2b,alpha/2,3);
IC_F_low2_up = quantile(Fn_low2b,1-(alpha/2),3);

IC_pi_IPV_low = quantile(pi_IPVb,alpha/2,3);
IC_pi_IPV_up = quantile(pi_IPVb,1-(alpha/2),3);
IC_pi_up_low = quantile(pi_upb,alpha/2,3);
IC_pi_up_up = quantile(pi_upb,1-(alpha/2),3);
IC_pi_low_low = quantile(pi_lowb,alpha/2,3);
IC_pi_low_up = quantile(pi_lowb,1-(alpha/2),3);
IC_pi_up2_low = quantile(pi_up2b,alpha/2,3);
IC_pi_up2_up = quantile(pi_up2b,1-(alpha/2),3);

IC_pi_cf_low = quantile(pi_cfb,alpha,3);

% CSV files
F_s = [Fn_IPV Fn_up Fn_low Fn_low2];
pi_s = [pi_IPV pi_low pi_up pi_up2];
ICF_up = [IC_F_IPV_up IC_F_up_up IC_F_low_up IC_F_low2_up];
ICF_low = [IC_F_IPV_low IC_F_up_low IC_F_low_low IC_F_low2_low];
ICpi_up = [IC_pi_IPV_up IC_pi_low_up IC_pi_up_up IC_pi_up2_up];
ICpi_low = [IC_pi_IPV_low IC_pi_low_low IC_pi_up_low IC_pi_up2_low];

tot = [r_grid F_s pi_s ICF_up ICF_low ICpi_up ICpi_low pi_cf IC_pi_cf_low];
csvwrite('graph_output.csv',tot)
toc



% ---------------------------------------------------------------
% Run F-test for whether fraction of mills is constant across 
% auctions with different number of bidders 
% ---------------------------------------------------------------
%% Getting and sorting data
data_=xlsread('CLSW_data.xls');

data = [data_(:,11) data_(:,1) data_(:,9) data_(:,6) data_(:,5) data_(:,4) ...
    data_(:,2) data_(:,3) data_(:,7) data_(:,8) data_(:,12)];

clear data_

data=data(data(:,3)<500,:);
data=data(data(:,1)<9,:);

Y = data(:,11);
k1 = max(data(:,1))-1;
X1 = zeros(length(data),k1);
obs = zeros(k1,1);

% creates inputs for table 1
for j=1:k1
    X1(:,j) = (data(:,1)==j+1);
    obs(j,1) = length(data(data(:,1)==j+1,1));
end
b1 = (X1'*X1)\(X1'*Y);

res1 = Y-X1*b1;
Var_b1 = (X1'*X1)\(X1'*diag(res1.^2)*X1)/(X1'*X1);
R1 = [1 -1 0 0 0 0 0; 1  0 -1 0 0 0 0; 1 0 0 -1 0 0 0; 1 0 0 0 -1 0 0; ...
    1  0 0 0 0 -1 0; 1  0 0 0 0 0 -1];
F1 = (R1*b1)'*((R1*Var_b1*R1')\eye(k1-1))*(R1*b1);
crit1 = chi2inv(0.95,k1-1);
if F1>crit1
    disp('reject :(')
else
    disp('fail to reject :)')
end

 
end

  
% ---------------------------------------------------------------
% % Bissecion method to invert (n-1)-th order statistic
% ---------------------------------------------------------------    
function f  = func1(tol,N,F)
% Bissecion method to invert (n-1)-th order statistic

l=0;
u=1;

x=0.5*(u+l);

crit=1;

while crit > tol
    inv = N*x^(N-1)-(N-1)*x^N-F;
    crit=abs(inv);
    if inv<0
        l=x;
        x=0.5*(u+l);
    elseif inv>0
        u=x;
        x=0.5*(u+l);
    end
end

f=x;

end    
    
    
    
    
    
